
text_ext_map = {
    'json': 'application/json',
    'js': 'application/javascript',
    'xml': 'application/xml',
    'sh': 'application/x-sh',
    'py': 'application/x-python',
    'php': 'application/x-httpd-php',
    'phtml': 'application/x-httpd-php',
    'latex': 'application/x-latex',
    'tex': 'application/x-tex',
    'htm': 'text/html',
    'html': 'text/html',
    'shtml': 'text/html',
    'talk': 'text/x-speech',
    'spc': 'text/x-speech',
    'rtf': 'text/rtf',
    'rtx': 'text/richtext',
    'txt': 'text/plain',
    'md': 'text/markdown',
    'markdown': 'text/markdown',
    'css': 'text/css',
    'csv': 'text/csv',
    'ics': 'text/calendar',
}
